# TKG MGMT deployment script
## Purpose: 
This script (deploy-mgmt-cluster.sh) installs a TKG v1.1.3 management cluster.  
This repo represents a point in time deploy and will evolve as newer versions of TKG are rleased/integrated.  
It uses a bootstrap single node kubernetes cluster running on top of docker (KinD) to provision the management cluster in the IAAS.   
This automation is writen to facilitate deployment into offline/airgapped environments.  

NOTE: These instructions are for a vSphere 6.7u3 or 7.x (without `vSphere with Kubernetes` enabled).  


# vSphere Airgap Deployment
### ENV Prereq:

<details><summary>vSphere Service Account</summary>
<a href=https://docs.vmware.com/en/VMware-Tanzu-Kubernetes-Grid/1.1/vmware-tanzu-kubernetes-grid-11/GUID-install-tkg-vsphere.html#vsphere-permissions>VMware Documentation</a>

Follow the linked documentation to create a service account with the appropriate permissions.  
NOTE: if using the import-ova.sh script, ensure the sevice account has appropriate permissions to the folder the templates will be uploaded to.

</details>

<details><summary>SSH key pair</summary>
<a href=https://docs.vmware.com/en/VMware-Tanzu-Kubernetes-Grid/1.1/vmware-tanzu-kubernetes-grid-11/GUID-install-tkg-vsphere.html#ssh-key>VMware Documentation</a>

If you do not already have an SSH key pair, you can use the steps below to create one.

```bash
ssh-keygen -t rsa -b 4096 -C "email@example.com"

ssh-add ~/.ssh/id_rsa

# Copy the contents of `~/.ssh/id_rsa.pub` to the variable VSPHERE_SSH_AUTHORIZED_KEY
```

</details>

<details><summary>Base OS Image Templates</summary>
<a href=https://docs.vmware.com/en/VMware-Tanzu-Kubernetes-Grid/1.1/vmware-tanzu-kubernetes-grid-11/GUID-install-tkg-vsphere.html#import-baseos>VMware Documentation</a>

```bash
# For manual import steps see the documentation linked above.

# For the following script, ensure the ./templates/vsphere-airgap-mgmt-config.tpl file is copied and populated in the GOVC section.
# Usage: ./import-ova.sh <ENV_EXPORT_FILE> <OVA_FILE> <VM_NETWORK>"
./import-ova.sh ~/vsphere-tkg-env ~/photon-3-kube-v1.18.6_vmware.1.ova VLAN-1234"
```

</details>

<details><summary>Load Balancer Image Template</summary>
<a href=https://docs.vmware.com/en/VMware-Tanzu-Kubernetes-Grid/1.1/vmware-tanzu-kubernetes-grid-11/GUID-install-tkg-vsphere.html#import-haproxy>VMware Documentation</a>


```bash
# For manual import steps see the documentation linked above.

# For the following script, ensure the ./templates/vsphere-airgap-mgmt-config.tpl file is copied and populated in the GOVC section.
# Usage: ./import-ova.sh <ENV_EXPORT_FILE> <OVA_FILE> <VM_NETWORK> (if network name contains spaces encapsulate in quotes 'VM Network')
./import-ova.sh ~/vsphere-tkg-env ~/photon-3-haproxy-v1.2.4-vmware.1.ova VLAN-1234"
```

</details>


<details><summary>Container registery with required images and tls endpoint</summary>
<a href=https://docs.vmware.com/en/VMware-Tanzu-Kubernetes-Grid/1.1/vmware-tanzu-kubernetes-grid-11/GUID-install-tkg-airgapped-environments.html#tkg-112>VMware Documentation</a>

This is a modified version of the script found in the VMware documentation to facilitate pulling the images from an internet connected jumpbox and pushing the images from and airgapped jumpbox.

```bash
# generate the ~/.tkg directory structure so the bom files are present
tkg get mc

# generate the pull script from an interned connected jumpbox
./registry/gen-publish-images-airgap.sh

# pull the images to the internet connected jumpbox
$HOME/pull-images.sh

# cd to home dir 
cd $HOME

# tar and gzip the directory containing the images, then move the tgz file to the airgapped jumpbox
tar -zcf ~/tkg-images-$(tkg version -o json | jq -r '.Version').tgz ./tkg-images-$(tkg version -o json | jq -r '.Version')

### from the airgapped jumpbox
# untar the images directory to home dir 
tar -zxf tkg-images-$(tkg version -o json | jq -r '.Version').tgz -C $HOME

# cd into the unpacked image files directory
cd $HOME/tkg-images-$(tkg version -o json | jq -r '.Version')

# Set an env for the registry
# e.g. generic registry myimagerepo.example.org
# e.g. harbor registry myimagerepo.example.org/<project_name> (the lowest level to store an image in harbor is a project)
export TKG_CUSTOM_IMAGE_REPOSITORY=<registry_fqdn>


# login to the private/airgapped registry
docker login ${TKG_CUSTOM_IMAGE_REPOSITORY}

# push the images to the airgapped regisry
./push-images.sh ${TKG_CUSTOM_IMAGE_REPOSITORY}

```

</details>

<details><summary>Root CA for PATH tls cert</summary>
Base64 encode the root ca or self signed certificate pem file for the airgapped registry and store in the environment variables file

```bash
cat <PEM_FILE> | base64 -w0
# store the base64 string as the value to AIRGAPPED_B64ENCODED_CA
```

</details>

### jumpbox Prereq:
- tkg == v1.1.3 cli in $PATH
- jq >= 1.6 cli in $PATH
- yq >= 3.4.0 cli in $PATH
- docker >= 18.09.7 in $PATH

### Optional:
- Vault server /vault cli in $PATH
- govc cli in $PATH

### Usage:
1) Populate an environment variables file using the template
```bash
$ cp ./templates/vsphere-airgap-mgmt-config.tpl ~/vsphere-tkg-env

# Add all of the reqired metadata for the deployment
$ vim ~/vsphere-tkg-env
```


2) Deploy the Management Cluster. 
```bash
# Usage: ./deploy-mgmt-cluster.sh <ENV_EXPORT_FILE>
./deploy-mgmt-cluster.sh ~/vsphere-tkg-env 

# NOTE: If you are deploying to vSphere 7.x and are prompted with the following question, type 'y' and press enter.
# You are about to provision a Kubernetes cluster on a vSphere 7.0 cluster that has not been optimized for Kubernetes. For the best experience of Kubernetes on vSphere, contact your vSphere administrator and ask them to enable the vSphere with Kubernetes feature.
# Do you want to proceed without enabling the vSphere with Kubernetes feature?:
```


3) Deploy a Workload Cluster.
```bash
# Usage: ./deploy-work-cluster.sh <ENV_EXPORT_FILE> <CLUSTER_NAME> --plan <PLAN_NAME> <OPTIONAL_FLAGS_AND_ARGS>
./deploy-work-cluster.sh ~/vsphere-tkg-env my-workload-cluster --plan dev --worker-machine-count 3
```