#! /bin/bash

function usage(){
    echo "Usage: ./deploy-work-cluster.sh <ENV_EXPORT_FILE> <CLUSTER_NAME> --plan <PLAN_NAME> <OPTIONAL_FLAGS_AND_ARGS>"
    echo "       ./deploy-work-cluster.sh ~/aws-tkg-env my-workload-cluster --plan dev"
    echo "       ./deploy-work-cluster.sh ~/aws-tkg-env my-workload-cluster --plan dev --worker-machine-count 3"
}

if [[ -z $1 ]]; then
    echo "Error: missing arg for env export file"
    usage
    exit 1
elif [[ ! -f $1 ]]; then
    echo "Error: env export file not found"
    exit 1
fi

if [[ $# -lt 4 ]]; then
    echo "Error: missing arg(s) for cluster creation"
    usage
    exit 1
fi

#abort on error or if vars not set
set -ue

# set envs 
source $1

# drop the first argument
shift

# get script directory for pathing
WORKING_DIR=$(dirname "$0")

# Substitute the variables in the containerd config.toml file, base64 encode and set as env
export TKG_CUSTOM_IMAGE_REPOSITORY_FQDN=$(echo ${TKG_CUSTOM_IMAGE_REPOSITORY} | cut -d'/' -f1)
export AIRGAPPED_B64ENCODED_TOML=$(envsubst < ${WORKING_DIR}/templates/containerd-config.toml | base64 -w0)

# Create workload cluster
tkg create cluster $@
